"use strict";

var isPlainObject = require("lodash.isplainobject");
var isName = require("./isName");
var isConsts = require("./isConsts");
var isParamValues = require("./isParamValues");
var isParamIndices = require("./isParamIndices");
var isUnits = require("./isUnits");
var isVariants = require("./isVariants");

function validate(synthdef, strict) {
  if (Array.isArray(synthdef)) {
    return synthdef.every(function (synthdef) {
      return _validate(synthdef, strict);
    });
  }
  return _validate(synthdef, strict);
}

function _validate(synthdef, strict) {
  if (!isPlainObject(synthdef)) {
    return false;
  }
  if (!isName(synthdef.name)) {
    return false;
  }
  if (!isConsts(synthdef.consts)) {
    return false;
  }
  if (!isParamValues(synthdef.paramValues)) {
    return false;
  }
  if (!isParamIndices(synthdef.paramIndices, synthdef.paramValues.length)) {
    return false;
  }
  if (!isUnits(synthdef.units, synthdef.consts.length)) {
    return false;
  }
  if (strict) {
    if (!isVariants(synthdef.variants)) {
      return false;
    }
  }
  return true;
}

module.exports = { validate: validate };