"use strict";

var isPlainObject = require("lodash.isplainobject");
var toValues = require("lodash.values");
var sum = require("lodash.sum");

function isParamIndices(value, numberOfParams) {
  if (!isPlainObject(value)) {
    return false;
  }
  var values = toValues(value);

  if (!values.every(isValidParamIndex)) {
    return false;
  }

  var slots = values.reduce(fillSlot, []);

  return slots.length === numberOfParams && sum(slots) === numberOfParams;
}

function isValidParamIndex(value) {
  if (!isPositiveInteger(value.index)) {
    return false;
  }
  if (!isPositiveInteger(value.length)) {
    return false;
  }
  return true;
}

function fillSlot(slots, _ref) {
  var index = _ref.index;
  var length = _ref.length;

  for (var i = index; i < index + length; i++) {
    slots[i] = slots[i] ? slots[i] + 1 : 1;
  }
  return slots;
}

function isPositiveInteger(value) {
  return Math.abs(value | 0) === value;
}

module.exports = isParamIndices;